#import <simtools.h>
#import <collections.h>
#import <random.h>
#import "ModelSwarm.h"
#import "Proposer.h"
#import "Candidate.h"
#import "Electioneer.h"
#import "Voter.h"
#import "MultiDimSpace.h"
#import "Parameters.h"
#import "CountHeads.h" //for voteDistribution access to counters
#import "Median.h"
#import "ObserverSwarm.h"

@implementation ModelSwarm  

+ createBegin: (id)aZone 
{
  ModelSwarm * obj;
  obj = [super createBegin: aZone];
  return obj;
}

- createEnd 
{
  return [super createEnd];
}


- setObserverSwarm: anObserver
{
  observerSwarm = anObserver;
  return self;
}

- getVoterList 
{
  return voterList;
}
   
- getWinnerList 
{
  return winnerList;
}

    
- getCandidateList 
{
  return candidateList;
}
 
- getElectioneer
{
  return anElectioneer;
}

- getMDSpace
{
  return mdSpace;
}


// buildObjects in light of the required numDimensions of the space.   
- buildObjects: (int) numDims
{
  int i;
  int voterType = getInt(arguments,"voterType");

  Point * initialPoint;
 
  [super buildObjects];  

  numDimensions = numDims;

  mdSpace = [MultiDimSpace createBegin: self];
 
  [mdSpace setNumDimensions: numDimensions];
  [mdSpace startRNG];
  mdSpace = [mdSpace createEnd]; 

  for(i=0; i < numDimensions; i++)
    {
      [mdSpace setMinDim: i To: 0];
      [mdSpace setMaxDim: i To: 100];
      [mdSpace setMeanDim: i To: 50];
      [mdSpace setVarianceDim: i To: 400];
    }
 

  //here we tell the model where to start, by placing the current point
  current = [mdSpace createPoint];
  for(i=0; i < numDimensions; i++)
    {
      [current setCoordinate: i To: [uniformUnsRand getUnsignedWithMin: 0 withMax: 100]];
    }
  
  initialPoint = [mdSpace createPoint];
  [initialPoint copyPositionFrom: current];
  
  [initialPoint setPixmap: 
		    [[[Pixmap createBegin: self] 
		       setFile: "initialPoint.png"] 
		      createEnd]];

  //This one gets used over and over again, so don't create a object
  //for it every time. Just create the Pixmap object once, and let
  //lots of points draw it.

  pastWinnerPixmap =    [[[Pixmap createBegin: self] 
			   setFile: "pastWinner.png"] 
			  createEnd];

  winnerPixmap =  [[[Pixmap createBegin: self] 
 			   setFile: "letterP.png"] 
 			  createEnd];


  winnerList = [List create: self ];
   
  [winnerList addFirst: initialPoint]; 
  
  proposalList = [List create: self ];// collection of offerings presented to voters
 
  voterList = [List create: self ];
   
  candidateList = [List create: self];
 
  for (i = 0; i < getInt(arguments,"numCitizens"); i ++) 
    {  
      Voter * aVoter;
      Point * aPoint = [mdSpace createPoint];
      [mdSpace assignPositions: aPoint Type: voterType];

      aVoter = [Voter createBegin: self];
      [aVoter setMDSpace: mdSpace];
      aVoter = [aVoter createEnd];
      [aVoter setObserverSwarm: observerSwarm];
      [aVoter setIdealPoint: aPoint];
      
      [aVoter setVoterColor: 0];
      [voterList addLast: aVoter];
      [aVoter setidnum: i+1];
      // Uncomment Following to get creation-state information to compare against voter list
      // printf immediately below
      // [aVoter sayHi];
    }

  //this is included if you want to verify the voter positions in the console output
  //   index=[voterList begin:self];
  //   while((aVoter=[index next])){
  //         [aVoter sayHi];
  //   [index drop];



  if ( getInt(arguments,"numCandidates")==1)
    {
      id aProposer;
      aProposer = [Proposer createBegin: self];
      [aProposer setMDSpace: mdSpace];
      [aProposer setID: 0 ]; 
      [aProposer setPixmapName: "proposal.png"];  
      aProposer = [aProposer createEnd];

      [aProposer setNumDimensions: numDimensions];
      
      [aProposer initMin: [(Parameters *)arguments getMinStep]
		 Max: [(Parameters *)arguments getMaxStep] 
		 Width:  [(Parameters *)arguments getWidthOfSearch] ];
      [candidateList addLast: aProposer];
    }

  else
    {
      for (i = 0; i < getInt(arguments,"numCandidates"); i++)
	{
	  id aCandidate;
	  char myName[15];
	  
	  aCandidate = [Candidate createBegin: self];
	  [aCandidate setMDSpace: mdSpace];
	  aCandidate = [aCandidate createEnd];	
	  [aCandidate setID: i+1]; 
	  sprintf(myName, "c%02d.png",i+1);
	  [aCandidate setPixmapName: myName];

	  	  

	  [aCandidate setNumDimensions: numDimensions];
	  [aCandidate initMin: [(Parameters *)arguments getMinStep]
		      Max: [(Parameters *)arguments getMaxStep] 
		      Width:  [(Parameters *)arguments getWidthOfSearch] ];


	  [candidateList addLast: aCandidate];
	}
    }
      
  anElectioneer = [Electioneer create: self];
  voteDistribution = [anElectioneer init: getInt(arguments,"numCandidates") ];

  
  theMedian = [mdSpace createPoint];
  [self calculateMedianList: voterList OutputTo: theMedian];

  printf("\n\nthe median is \n \n");
  [theMedian sayHi];
 
  return self; 
}


//plot proposals as well as winneres
// - grabProposals

// {
//   unsigned j;
//   for (j = 0; j < [candidateList getCount]; j++)
//     {
//       id aCandidate = [candidateList atOffset: j ];
//       [winnerList addFirst: [aCandidate getProposal]];
//     }
//   return self;
// }
// if there is only 1 candidate, the proposalList is supposed to include
// the status quo against 1 candidate.

// if there are more candidates, the status quo is never supposed to appear
// as a proposal, but it is still used as a baseline against which these
// silly myopic candidates frame their proposals.

- getProposal
{
  id anObject;
  id index;

  if ( [candidateList getCount] == 1 && [proposalList getCount] > 0) //don't delete current
    {
      [proposalList removeFirst];
    }

  [proposalList removeAll];
  
  if ([candidateList getCount] == 1) //put current back on
    [proposalList addLast: current];
  

  index = [candidateList begin: self];
  for ( anObject = [index next]; [index getLoc]==Member; anObject=[index next])
    {
      Point * proposal = [anObject makeProposalAgainst: current];
      [proposalList addLast: proposal]; 
    }
  [index drop];

  return self;
}


- evaluateProposal
{
  Proposer * winningProposer;
  int chosenAlternative = [anElectioneer  evaluate: proposalList L: voterList];

  // This causes the winner positions to be put onto a list
  // called "winnerList" which is then displayed. 
 
  
  if ([candidateList getCount] == 1)
    {
      if (chosenAlternative != 0) //sq did not win
	{
	  Point * pastWinner = nil;
	  Point * winner = nil;
	  winningProposer = [candidateList atOffset: 0];
	  winner  = [winningProposer removeProposal];
	
	  [current copyPositionFrom: winner];
	  [winner setPolicyColor: 2]; //2 is green
	  [winner setPixmap: winnerPixmap]; 
	  if (getCurrentTime()> 1 && (pastWinner = [winnerList removeLast] )!= nil)
	    {
	      [pastWinner setPixmap: pastWinnerPixmap];
	      [winnerList addLast: pastWinner];
	    }
	  [winnerList addLast: winner]; 
	  [winningProposer  receiveResult: YES
					Votes: [[voteDistribution atOffset: 1] getCnt]];
	}
#ifdef NO_PIXMAP
      else  //color loser red, add to list
	{
	  Point * loser = [proposalList removeLast];
	  [[candidateList atOffset: 0]  receiveResult: NO
					Votes: [[voteDistribution atOffset: 1] getCnt]];
	  [loser setPolicyColor: 3];
	  [winnerList addLast: loser];
	}
#endif 
    }
  
  else
    {
      unsigned int i;
      id pastWinner;
      Point * winner = nil;
      winningProposer = [candidateList atOffset: chosenAlternative];
  
      winner = [winningProposer removeProposal];
      [winner setPolicyColor: 2];

      if (getCurrentTime()> 0 && (pastWinner = [winnerList removeLast] )!= nil)
	{
	  [pastWinner setPixmap: pastWinnerPixmap];
	  
	  [winnerList addLast: pastWinner];
	}
     
      [current copyPositionFrom: winner];
      [winnerList addLast: winner];
      
      for (i = 0; i < [candidateList getCount]; i++)
	{
	  id aCandidate = [candidateList atOffset: i ];
	  [aCandidate  receiveResult: ((int)i == chosenAlternative) 
		       Votes: [[voteDistribution atOffset: i ] getCnt] ];
	}
    }

  return self;
  
}


// This sweeps through the object that we want to draw 
// themselves on the rasters, and it tells them to draw themselves.  
- drawAgents
{
  id index;
  id anObject;
  id previous = nil;
  int i;

  index = [voterList begin: self];
  for ( anObject = [index next]; [index getLoc]==Member; anObject=[index next])
    {
      [anObject drawSelfOn: observerSwarm];//??last argument unnecessary
    }
  [index drop];

  index = [winnerList begin: self];
  for ( anObject = [index next]; [index getLoc]==Member; anObject=[index next])
    {
      id rasters = [observerSwarm getRasters];
      [anObject drawSelfOn: observerSwarm];
      
      
      if ( previous )
	{
	  for (i = 0; i < (numDimensions+1)/2; i++)
	    { 
	      int j;
	      for (j = 0; j < 2; j++)
		{
		  [[rasters atOffset: i] lineX0: (int)[previous getCoordinate: (2*i)] 
					 Y0: (int)[previous getCoordinate: (2*i)+1]
					 X1: (int)[anObject getCoordinate: (2*i)] 
					 Y1: (int)[anObject getCoordinate: 2*i+1] 
					 Width: 1 
					 Color: 2];
		}
	    }
	}
      
      previous = anObject;
    }

  [index drop];

  index = [candidateList begin: self];
  for ( anObject = [index next]; [index getLoc]==Member; anObject=[index next])
    {
      Point * aPoint = [anObject getProposal];
      if (aPoint) [aPoint drawSelfOn: observerSwarm];
    }
  [index drop];

  return self;
}

- buildActions 
{
  id modelActions;  
  //Create an ActionGroup to hold the messages

  modelActions = [ActionGroup create: self];
  [modelActions createActionTo: self  message: M(getProposal)];
  
  
  [modelActions createActionTo: self message: M(evaluateProposal)];
  
  modelSchedule = [Schedule createBegin: self];
  [modelSchedule setRepeatInterval: 1];
  modelSchedule = [modelSchedule createEnd];
  [modelSchedule at: 0 createAction: modelActions]; 

  return self;

}

- activateIn: (id) swarmContext 
{
  [super activateIn: swarmContext];
  [modelSchedule activateIn: self];
  
  return [self getActivity];
}


- (Point *)getMedian
{
  return theMedian;
}



- (double)getDistFromMedian
{
  double dist = [theMedian distanceFrom: [winnerList getLast]];
  // fprintf (stderr,"dist from median %f \n",dist);

  if (distFromMedian == 0) distFromMedian = dist;
  else 
    {
      distFromMedian = 0.1*dist + 0.9*distFromMedian; 
    }
  return distFromMedian;
}


- (BOOL)calculateMedianList: list OutputTo: (Point *)outputPoint
{
  int i,k,j;
  int numOfMemberPositions;
  id index;
  id aMember;
  id aPosition; 
  id <List> listOfPoints;
  double median[numDimensions];
 
  listOfPoints = [List create: [self getZone]];

  index = [list begin: [self getZone]]; 

  while((aMember=[index next]) != NULL)
    {
      if ((aPosition = [aMember getIdeal]) != NULL) 
	{
	  [listOfPoints addLast: aPosition];
	}
    }
  [index drop];

  if( (numOfMemberPositions = [listOfPoints getCount]) ==0 )
    {
      printf("FOUND AN EMPTY listOfPoints, median=old position  \n");
      //Explanation: If nobody in the group has an opinion about this issue,
      // tell it so with a NO!  Should not erase that position yet, though,
      // because we still need to do some bookkeeping.
      return NO;
    }

  if (numOfMemberPositions%2 == 0) k = (numOfMemberPositions/2)-1;

  else k = -1+((numOfMemberPositions+1)/2); 

  for( j=0; j < numDimensions; j++ )
    {
      double idealArray[numOfMemberPositions];
	        
      for (i=0; i< numOfMemberPositions; i++) 
	{
	  idealArray[i] = [[listOfPoints atOffset:i] getCoordinate:j]; 
	}
      //printf("The median Space: %d, Dim: %d: %f \n", s,j,findMedian(k,numOfMemberPositions,idealArray));
      if(numOfMemberPositions%2 == 1)
	median[j]=sedgewick_select(idealArray, 0, numOfMemberPositions-1, k);
      else
	median[j]=0.5*(sedgewick_select(idealArray, 0, numOfMemberPositions-1, k)+
		       sedgewick_select(idealArray, 0, numOfMemberPositions-1, k+1) );
      //      printf("According to sedgewick_select, median[%d] is %f\n",j,median[j]);
       //	if((numOfMemberPositions%2 == 1))	
       //          median[j]=findMedian(k,numOfMemberPositions,idealArray);
       //	else

       //	median[j]=0.5*(findMedian(k,numOfMemberPositions,idealArray)+ idealArray[k+1]);          
       //printf("According to findmedian, median[%d] is %f\n",j,median[j]);
	
    }
  [outputPoint setPos: median];
  [listOfPoints drop];
  return YES;
}




@end



  


