//MultiDimSpace.m


#import "MultiDimSpace.h"
#import <math.h>

#import <misc.h>
#import <random.h>
#import <collections.h>
#import "Point.h"



@implementation MultiDimSpace

- createEnd
{
  [super createEnd];
  
  if (numDimensions <= 0) raiseEvent(InvalidArgument, "numDimensions wrong\n");

  minValue = [[self getZone] allocBlock: (numDimensions * sizeof(double))];
  
  maxValue = [[self getZone] allocBlock: (numDimensions * sizeof(double))];

  meanValue = [[self getZone] allocBlock: (numDimensions * sizeof(double))];
 
  meanValue =[ [self getZone]  allocBlock: (numDimensions * sizeof(double)) ];

  varianceValue = [[self getZone] allocBlock: (numDimensions * sizeof(double))];
  
  return self;
}


- (void)setNumDimensions: (int)x
{
  numDimensions = x;
}


- (int)getNumDimensions
{
  return numDimensions;
}

- setMinDim: (int)dim To: (double)value
{
  minValue[dim]=value;
  return self;
}

- setMaxDim:  (int)dim To: (double)value
{
  maxValue[dim]= value;
  return self;
}

- setMeanDim:(int)dim To: (double)value
{
  meanValue[dim]= value;
  return self;
}

- setVarianceDim:  (int)dim To: (double)value
{
  varianceValue[dim]= value;
  return self;
}

- (double)getMinDim: (int)dim
{
  return minValue[dim];
}

- (double)getMaxDim: (int)dim   
{
  return maxValue[dim];
}

- (double)getMeanDim: (int)dim   
{
  return meanValue[dim];
}

- (double)getVarianceDim: (int)dim
{
  return varianceValue[dim];
}


- (void)startRNG
{
  
  id newGenForNormal = [C2TAUS3gen create: [self getZone]  
					  setStateFromSeed: [randomGenerator getUnsignedSample]];
  aNormalDist = [NormalDist create: [self getZone] 
			       setGenerator: newGenForNormal];
  
  id newGenForUniform = [MT19937gen create: [self getZone] 
			   setStateFromSeed: [randomGenerator getUnsignedSample]];

                       
  aUniformDist = [UniformDoubleDist create: [self getZone]
				    setGenerator: newGenForUniform
				    setDoubleMin: 0
				    setMax: 1];
}



- (Point *)createPoint
{
  Point * location;
  location = [Point createBegin: [self getZone]];
  location = [location createEnd];
  [location setNumDimensions: numDimensions];
  [location buildObjects];
  
  return location;
}


// Types: 0 = normal
// 1 = uniform
// 2 = bimodal
// 

- (Point *)assignPositions: (Point *)location Type: (int)spaceType
{
 
  int i = 0; 

  switch (spaceType)
    {
    case 0:
      {	
	for (i=0; i < numDimensions ; i++)
	  {
	    [location setCoordinate: i To: [aNormalDist  getSampleWithMean: meanValue[i]  withVariance: varianceValue[i]]];
	  }

	break;
      }
      

    case 1:
      {
	for (i=0; i < numDimensions ; i++)
	  {
	    [location setCoordinate: i To: [aUniformDist  getDoubleWithMin: minValue[i]  withMax: maxValue [i]]];
	  }
	break;
      }


    case 2:
      {
	int indicator = 0;
	double holderfloat = [aNormalDist getSampleWithMean: 70 withVariance: 100];
	double ideology = 999;
	if ([aUniformDist getDoubleSample] < .5) indicator = 0; else indicator = 1;
	if (indicator == 1) ideology = holderfloat;
	if (indicator == 0) ideology = 100-holderfloat;

	for (i=0; i < numDimensions ; i++)
	  {
	    [location setCoordinate: i To: ideology+[aNormalDist getSampleWithMean: 0 withVariance: 100]];
	  }
	break;
      }


    case 3:
      {
	double value[numDimensions];
	for (i=0; i < numDimensions ; i++)
	  {
	    value[i] =  [aNormalDist getSampleWithMean: meanValue[i] withVariance: varianceValue[i]] ;
	    [location setCoordinate: i To: value[i]];
	  }

	if ( value[0] <= 0.5 * (maxValue[0]-minValue[0]) )
	  {
	    if ( value[1] > 0.5 * (maxValue[1]-minValue[1]) )
	      { ; }
	    else
	      {
		if ( [aUniformDist getDoubleWithMin: 0 withMax: 1] < 0.5)
		  {
		    
		    [location setCoordinate: 0 To: 50 + (50 - value[0]) ] ;
		  }
		else
		  {
		    [location setCoordinate: 1 To: 50 + (50 - value[1]) ] ;
		  }
		
	      }
	  }
      	break;
      }



    default: 
      raiseEvent (InvalidArgument, "In MultiDimSpace assignPositions, the spaceType is invalid. Ty: %d",spaceType);
    }
  
  fprintf(stderr,"spaceType=%d pos %f\n",spaceType,[location getCoordinate:i]);
  return location;
}


@end   
   
