#import <objectbase.h>

#import "Parameters.h"

#import <misc.h>  //for atoi

id
makeProbe (id obj, const char *ivarName)
{
  id probe = [VarProbe createBegin: [obj getZone]];
  [probe setProbedClass: [obj getClass]];
  [probe setProbedVariable: ivarName];
  return [probe createEnd];
}


int
getInt (id obj, const char *ivarName)
{
  id probe = makeProbe (obj, ivarName);
  int ret = [probe probeAsInt: obj];
  [probe drop];
  return ret;
}

@implementation Parameters

+ createBegin: aZone
 {
   static struct argp_option options[] = {
     {"inputfile", 'I',"filename",0,"set fn",5},
     {"run",'R',"RunNumber",0,"Run is...",6},
     {"numCitizens",'n',"N", 0, "num citizens", 8},
     {"numCandidates",'g',"numCandidates",0,"num candidates",9},
     {"voterType",'V',"voterType",0,"voter type",10},
     {"duration",'D',"Duration",0,"stop/pause at",11},
     {"minStep",'e',"minStep",0,"smallest allowed step",12},
     {"maxStep",'x',"maxStep",0,"largest allowed step",13},
     { 0 }
   };

   Parameters *obj = [super createBegin: aZone];
   [obj addOptions: options];

   obj->numDimensions = 5;
   obj->numCitizens = 555;
   obj->numCandidates = 1;
   obj->run=time(0);
   obj->voterType = 1;
   obj->duration=50;
   obj->maxStep = 4.0;
   obj->minStep = 0.0;
   obj->widthOfSearch = 2;


   return obj;
 }



- init
{
  id probeMap;
  probeMap = [EmptyProbeMap createBegin: [self getZone]];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];
  // Add in a bunch of variables, one per simulation parameter
  [probeMap addProbe: [probeLibrary getProbeForVariable: "numCitizens"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "numCandidates"
                                    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "numDimensions"
				    inClass: [self class]]];

  [probeMap addProbe: [probeLibrary getProbeForVariable: "voterType"
				    inClass: [self class]]];            			                                       
  [probeMap addProbe: [probeLibrary getProbeForVariable: "maxStep"
				    inClass: [self class]]]; 
  		                                       
  [probeMap addProbe: [probeLibrary getProbeForVariable: "minStep"
				    inClass: [self class]]]; 
    
  [probeMap addProbe: [probeLibrary getProbeForVariable: "widthOfSearch"
				    inClass: [self class]]];                                                                    

 
  [probeMap addProbe: [probeLibrary getProbeForVariable: "run"
                                    inClass: [self class]]];
 
  [probeMap addProbe: [probeLibrary getProbeForVariable: "duration"
                                    inClass: [self class]]];
  [probeLibrary setProbeMap: probeMap For: [self class]];
  return self;
}



- (int)parseKey: (int) key arg: (const char*) arg
 {
   if (key == 'n')
     {
       if(arg)
	 numCitizens = atoi(arg);
       return 0;
     }
   else if (key == 'g')
     {
       if(arg)
	 numCandidates = atoi(arg);
       return 0;
     }
   
   else if (key == 'D')
     {
       if(arg)
	 duration = atoi(arg);
       return 0;
     }
   
  else if (key == 'V')
     {
       if(arg)
	 voterType = atoi(arg);
       return 0;
     }
   
   else if (key == 'e')
     {
       if(arg)
	 minStep = strtod(arg, 0);
       return 0;
     }
      
   else if (key == 'x')
     {
       if(arg)
	 maxStep = strtod(arg, 0);
       return 0;
     }
   
   else
     return [super parseKey: key arg: arg];
 }



- (double)getWidthOfSearch
{
  return widthOfSearch;
}

- (double) getMaxStep
{
  return maxStep;
}

- (double)getMinStep
{
  return minStep;
}
@end










