//Point.m


#import "Point.h"
#import <math.h>
#import "ModelSwarm.h"
#import "ObserverSwarm.h"
#import <misc.h>
#import "Parameters.h"


@implementation Point

- createEnd
{
  policyColor = 3;  //default color = red
  return [super createEnd];
}



- buildObjects
{
  int i;

  positn = [[self getZone] allocBlock: numDimensions * sizeof(double)];
  
  for ( i = 0; i <  numDimensions; i++ )
    {
      positn[i] = 0;
    }
  return self;
}


- (void)setNumDimensions: (int) x
{
  numDimensions = x;
  NOFSPACES = (numDimensions+1)/2;
}

- (int)getNumDimensions
{
  return numDimensions;
}
    
- setPolicyColor: (int) c
{
  policyColor = c;
  return self;
}

- (void)drop
{   

  [[self getZone] freeBlock: positn blockSize: numDimensions * sizeof(double)];

  [super drop];
}


- (int)getSpaceSize
{
  return numDimensions;
}

- setPos: (double[])loc 
{
  int i;
  for (i=0; i <  numDimensions ; i++)
    {
      positn[i] = loc[i];
    }
  return self;
}

- setCoordinate: (int)dim To: (double)number 
{ 
  positn[dim] = number;
  return self; 
}


- copyPositionFrom: (Point *)inputPoint
{
  int i;
  double * positionPtr = [inputPoint getArray];

  if([inputPoint getSpaceSize] != numDimensions) 
    {
      raiseEvent( InvalidArgument, " incoming space size > existing \n");
      exit(0);
    }
  for(i=0; i<  numDimensions; i++)
    {
      positn[i] = positionPtr[i];
    }
  return self;
}


- (double)getCoordinate: (int)dim 
{
  // if (dim >=  numDimensions ) raiseEvent( InvalidArgument, "index > Dimension \n");
 
  if (dim < numDimensions)
     return positn[dim];
  else return 0;
}

- (BOOL)getCoordinates: (double[])dblArray 
                      From: (int) start To: (int) end 
{
  int i;
  
  if ( !dblArray ) {
    raiseEvent( WarningMessage, "dblArray is nil ?\n" );
    return NO;
  }

  //       if ( start > end ) {
  //          raiseEvent( WarningMessage, "start > end ?\n" );
  //          return FALSE;
  //       }
  
  //       if ( sizeof(dblArray) / sizeof( double ) <= end - start ) {
  //          raiseEvent( WarningMessage, "dblArray is too small?\n" );
  //          return FALSE;
  //       }
  
  for ( i = start; i <= end; i++ ) {
    positn[ i ] = dblArray[ i ];
  }
  
  return YES;
}

   
- (double *)getArray
{
  return positn;
}
 

- (double)distanceFrom: (Point*) pol 
{
  int m;
  double * coordinates;
  double newvalue;

  coordinates = [pol getArray];

  newvalue = 0;
  for (m=0; m <  numDimensions; m++)
    {
      newvalue += (positn[m]- coordinates[m])*(positn[m]- coordinates[m]);
    }
  return sqrt(newvalue); 
}


- sayHi
{
  int i;
  for(i=0; i< numDimensions; i++)
    {
      fprintf(stderr," %f: ", positn[i]);
    }
  printf("\n");
    
  return self; 
}


- drawSelfOn: r {
  int i;
  id rasters = [r getRasters];
  for (i=0; i < NOFSPACES ; i++)
    {
      [self drawPositionOn: [rasters atOffset: i] Space: i];
    }
  return self;
}


//if polPixmap already exists, drop it.
- (void)setPixmap: (id <Pixmap>)aPix
{
  polPixmap = aPix;
}

- (id <Pixmap>)getPixmap
{
  return polPixmap;
}


- drawPositionOn: r Space:(int)space
{
  //  printf("draw position on space: %d\n", space);
#ifdef N_PIXMAP
   [r drawPointX:  (int)[self getCoordinate:space*2]  Y:(int) [self getCoordinate:1+space*2] Color: policyColor];

#else
   /*double zoomFactorCorrection = 0.5/(double)[r getZoomFactor];*/
   if (polPixmap)
     {
       [polPixmap setRaster: r];
       
       [r draw:  polPixmap  X: (int) [self getCoordinate: space*2]/* + zoomFactorCorrection*[polPixmap getWidth]*/  Y:(int) [self getCoordinate: 1+space*2] /*+ zoomFactorCorrection*[polPixmap getHeight]*/];
     }
   else
     [r drawPointX: [self getCoordinate:space*2] /*+ zoomFactorCorrection*[polPixmap getWidth]*/   Y: [self getCoordinate:1+space*2] /*+ zoomFactorCorrection*[polPixmap getHeight]*/ Color: policyColor];
     //[r drawPointX: (int)[self getCoordinate:space*2]   Y: (int)[self getCoordinate:1+space*2] Color: policyColor];
#endif

  return self;
}



@end   
   




