#import "Proposer.h"
#import <simtools.h>
#import <math.h>
#import <random.h>
#import "Point.h"
#import "MultiDimSpace.h"
#import "Parameters.h"
#import <gui.h>  //for Pixmap

@implementation Proposer

// These are macros to make it easier to edit notation for
// use of Point objects, especially theta, the direction of movement.
// To get the i't coordinate from a point called "theta", do:
//   GET(theta,i)
// And to set the value to x, do
//
//   SET(theta,i,x)
// Otherwise, you have to do get the coordinate at offset 1 with 
// [theta getCoordinate: i] and set with [theta setCoordinate: i To: x];


#define GET(obj, iii) ([obj getCoordinate: iii])
#define SET(obj, iii, vvv)  ([obj setCoordinate: iii To: vvv])

- setMDSpace: (id)w  
{
  mdSpace = w;
  return self;
}

- (void)setNumDimensions: (int)x
{
  numDimensions = x;
}

- (void)setID: (int)x
{
  idNumber = x;
}



- createEnd 
{
  return [super createEnd];
}

- initMin: (double) min Max: (double)max Width: (double)width
{
  [self startRNG];

  [self setMaxStep: max];
  [self setMinStep: min];
  [self setWidthOfSearch: width];


  [self createInitialPosition];
  [self createInitialDirection];
  return self;
}


- (void) startRNG 
{
  
  id  aGenerator = [SWB1gen create: [self getZone] 
			    setStateFromSeed: [randomGenerator getUnsignedSample]];
  aNormalDist= [NormalDist create: [self getZone] 
			   setGenerator:  aGenerator
			   setMean: 0 setVariance: 1];
 
  id newGenForUniform = [MT19937gen create: [self getZone] 
				    setStateFromSeed: [randomGenerator getUnsignedSample]];

                       
  aUniformDist = [UniformDoubleDist create: [self getZone]
				    setGenerator: newGenForUniform
				    setDoubleMin: 0
				    setMax: 1];  
} 
   
- sayHi 
{
  printf("Hi, I'm a Proposer object\n");
  return self;
}


- createInitialPosition
{
  myProposal = [mdSpace createPoint];
  [myProposal setPixmap: polPixmap];
  [mdSpace assignPositions: myProposal Type:  getInt(arguments,"voterType")];
  return self;
}


- createInitialDirection 
{
  int i;

  theta = [mdSpace createPoint];
  goodTheta = [mdSpace createPoint];

  for (i=0; i<numDimensions; i++)
    {
      double x = [aUniformDist getDoubleWithMin: -2 withMax: +2];
      SET(theta, i, x);
      SET(goodTheta, i, x);
    }

  fprintf(stderr,"THETA");
  [theta sayHi];
  fprintf(stderr,"GOODTHETA");
  [goodTheta sayHi];

  return self;
}





- setMinStep: (double)value 
{
  minStep = value;
  return self;
}



- setMaxStep: (double)value 
{
  maxStep = value;
  return self;
}


// widthOfSearch: number of radians across which search is conducted.
// If the width represents a "full circle", then widthOfSearch=2,
// meaning the direction can change 2*pi radians.  If widthOfSearch=1,
// then direction changes are limited to a half space pointing in
// direction theta.
- setWidthOfSearch: (double)value 
{
  widthOfSearch = value;
  return self;
}   


// begin at position x.  Given a previous direction of "theta", 
// adopt a new direction and step to arrive at coordinates
// given in the point p.  The results of the calculation are
// passed by reference back to the caller, in the variable p.

- (Point *) makeProposalAgainst:  (Point *)x 
{
  double dist = [aUniformDist getDoubleWithMin: minStep withMax: maxStep];
  if (myProposal == nil) 
    {
      myProposal = [mdSpace createPoint];
      [myProposal setPixmap: polPixmap];
    }
 
  [self drawProposalFrom: x To: myProposal Width: widthOfSearch Distance: dist];
 
  fprintf(stderr,"proposal sayhi\n");[myProposal sayHi];


  return myProposal;;
}


- (void)setPixmapName: (char *)aname
{
  polPixmap = [Pixmap createBegin: [self getZone]];
  [polPixmap setFile: aname];
  polPixmap = [polPixmap createEnd]; 
}





// Choose a new point by going from a given point in a hopefully
// promising direction.  It is done by trigonometry, beginning at a
// current point, choosing a new direction of movement, theta (a
// variation on the last successful direction, goodTheta) and then
// calculating the new point proposal by projecting from the old
// position to the new position in the direction theta.

// I keep forgetting the trigonometry. Maybe you do too.
// If the space is 2 dimensional, then theta[0] is all that is needed to
// record the direction in which one is moving.  Given a current position,
// say (x,y), one needs to calcualte (newx,newy). Given direction,
// theta[0] and distance to be moved (dist), one can calculate a new 
// position.  

//    newx = x + dist*(cos(pi*theta[0]));
//    newy = y + dist*(sin(pi*theta[0])); 

// If space is 3D, then theta[0] and theta[1] are needed because there
// are two "angles" of movement in a 3D space. With 4D, there are 3
// angles.  And so forth. For an N dimensional space, there are N-1
// angles, or directions of movement. The first N-1 coordinates of
// theta, theta[0]...theta[n-2], are used to record the direction of
// movement.  Since theta is created as a "Point" object, it has space
// to store N coordinates.  That means the direction information is
// using up N-1 of the coefficients of the Point, and the last one,
// the N-1'th one, can be used to store the distance of the move.

// On average, the algorithm here finds new proposals by going in the
// direction of the last successful proposal. That's goodTheta. The
// new direction, theta, is perturbed at random.  The magnitude of the
// perturbation is determined by the withOfSearch parameter.  If
// widthOfSearch=2, then the perturbation is at its greatest, and
// proposals anywhere in a 360 degree circle can arise. On the other
// hand, if widthOfSearch is 1, then the new proposal is restricted to
// lie in the same direction, roughly speaking (1 radian is half of
// the space).

// I got the formula used here to translate theta and distance to
// calculate the change from one point (current) to a new offering
// (newPoint) from a math book.  The calculation is somewhat ugly
// because it has to work for any number of dimensions. It also is
// carrying along the adjustable "widthOfSearch" movement concept, so
// we have to select and scale theta properly, and also we have to
// make sure the proposed point stays inside the multidimensional
// space. When the newPoint goes outside the allowed space, then the
// proposal is scaled back proportionally.  Usually that restriction
// does not come into play, unless the current point happens to be
// near the edge.


- drawProposalFrom: (Point *)current To: (Point *)newPoint Width: (double)width Distance: (double)dist
{
  int k;
  double diff;

  double * positn = [current getArray];
 
  //function used to calculate the coordinates for new offerings as function of
  //current position, direction of movement (theta[j],j=0,...,n-2), and distance (theta[n-1])
  double
    sinProduct( int limit, Point * nthet )
    {
      int i;
      double value = 1;
      for(i=0; i < limit+1; i++)
        {
	  value = value*sin(M_PI*GET(nthet,i));
	  // printf("i= %d value %f sin %f \n",i, value, sin(M_PI*nthet[i]));
        }
      return value;
    }



  fprintf(stderr,"distance is %f\n", dist);

  //calculate a new direction, "theta". That's coefficients 0 through numDimensions-1;
  //theta is the angle in which movement will occur. If a full range of movement
  //is allowed (width=2), then the change in theta[i] should range from -1 to +1.
  for ( k = 0; k < numDimensions-1; k++ )
    {
      //find a diff value between -2 and + 2
      while( (diff=[aNormalDist getDoubleSample]) <= -2 || (diff >= 2) );
     
      //scale "diff" to change the direction, using width to scale.
      //theta[k] = goodTheta[k] + (diff / 4.0) * width;
      // SET(theta, k, GET(goodTheta,k) +  (diff / 4.0) * width );
      SET (theta, k, GET(goodTheta,k) + [aUniformDist getDoubleWithMin: -0.5*width withMax: 0.5*width]);
      printf("goodTheta: %f  theta: %f distance: %f \n", GET(goodTheta,k), GET(theta,k), dist);
    }


  // I cut out all the code the blocked the proposal from going outside the MDSpace.
  // Why not let those proposals get offered up?
  
  double newpos = positn[0] + dist*(cos(M_PI*GET(theta,0)));
  printf ("top of while \n");
  
  //[newPoint setCoordinate: 0 To: newpos];
  SET(newPoint,0,newpos);
  
  //already calculated first dimension. Now do rest
  if(numDimensions > 2)
    {
      for(k=1; k < numDimensions-1; k++)
	{
	  newpos = positn[k] + dist*sinProduct(k-1,theta)*cos(M_PI*GET(theta,k));
	  //[newPoint setCoordinate: k To: newpos];
	  SET(newPoint,k,newpos);
	}
    }
  
  
  // calculate the last directional coordinate.
  newpos = positn[numDimensions-1]+ dist*sinProduct(numDimensions-2,theta);
  //[newPoint setCoordinate: numDimensions-2 To: newpos];
  SET(newPoint,numDimensions-1,newpos);
  
  SET(theta,numDimensions-1,dist);
     
  
  return self;
}


//only if you won should you adopt the last direction
//as the new good direction
- (void)receiveResult: (BOOL)win Votes: (int)votes
{
  if (win == YES)
    {
      [goodTheta copyPositionFrom: theta];
    }
}

- (Point *)getProposal
{
  return myProposal;
}


- removeProposal
{
  Point * temp = myProposal;
  myProposal =  [mdSpace createPoint];
  [myProposal setPixmap: polPixmap];
  [myProposal copyPositionFrom: temp];
  return temp;
}


#undef SET
#undef GET

@end
